@@ -34,11 +34,6 @@ class MiddlewareBuildError(LihilError):
         super().__init__(f"Unable to instantiate middleware from {factory}")
 
 
-class InvalidParamTypeError(LihilError):
-    ...
-
-
-
 class NotSupportedError(LihilError):
     "A generic error for behaviors we currently do not support"
 
@@ -46,11 +41,18 @@ class NotSupportedError(LihilError):
         super().__init__(msg)
 
 
-class InvalidMarkTypeError(LihilError):
-    def __init__(self, mark_type: Any):
-        super().__init__(f"Invalid mark type {mark_type}")
+class InvalidParamError(LihilError): ...
+
+
+class InvalidParamPackError(InvalidParamError): ...
 
 
 class MissingDependencyError(LihilError):
     def __init__(self, dep_name: str) -> None:
         super().__init__(f"{dep_name} is required but not provided")
+
+
+class InvalidParamSourceError(LihilError):
+    def __init__(self, source: str, param_sources: tuple[str, ...]):
+        msg = f"Invalid source {source}, expected one of {param_sources}"
+        super().__init__(msg)
