@@ -16,4 +16,4 @@ class IEndpointInfo(Protocol, Generic[P, R]):
 
 
 class IPlugin(Protocol):
-    def __call__(self, endpint_info: IEndpointInfo[P, R]) -> IAsyncFunc[P, R]: ...
+    def __call__(self, endpoint_info: IEndpointInfo[P, R], /) -> IAsyncFunc[P, R]: ...
