@@ -5,21 +5,24 @@ from typing import (
     Any,
     Callable,
     ClassVar,
+    Generic,
     Literal,
     Mapping,
-    TypeAliasType,
+    TypeVar,
     cast,
     get_args,
+    get_origin,
 )
 
+from typing_extensions import TypeAliasType
 
 from lihil.constant import status as http_status
-from lihil.interface import ParamLocation, Record, lhl_get_origin
+from lihil.interface import ParamSource, Record, T
 from lihil.interface.problem import DetailBase, ProblemDetail
-from lihil.utils.json import encode_json
+from lihil.utils.json import encoder_factory
 from lihil.utils.string import to_kebab_case, trimdoc
-from lihil.utils.typing import all_subclasses, is_union_type
-from lihil.vendors import Request, Response #, WebSocket
+from lihil.utils.typing import all_subclasses, is_union_type, lenient_issubclass
+from lihil.vendors import Request, Response  # , WebSocket
 
 """
 Unlike starlette, only sync error handler is allowed
@@ -27,9 +30,10 @@ user should just return response, we don't put it in threadpool as well.
 
 If they want to do other things, do it with message bus
 """
+Exc = TypeVar("Exc")
 
-type ExceptionHandler[Exc] = Callable[[Request, Exc], Response]
-type ErrorRegistry = MappingProxyType[
+ExceptionHandler = Callable[[Request, Exc], Response]
+ErrorRegistry = MappingProxyType[
     "type[DetailBase[Any]] | http_status.Status", ExceptionHandler[Any]
 ]
 
@@ -37,12 +41,13 @@ type ErrorRegistry = MappingProxyType[
 def parse_exception(
     exc: type["DetailBase[Any]"] | TypeAliasType,
 ) -> type["DetailBase[Any]"] | int | list[type["DetailBase[Any]"] | int]:
-    exc_origin = lhl_get_origin(exc)
+    exc_origin = get_origin(exc)
 
     if exc_origin is None:
-
-        if isinstance(exc, type) and issubclass(exc, HTTPException):
+        if lenient_issubclass(exc, HTTPException):
             return exc
+        elif http_status.is_status(exc):
+            return http_status.code(exc)
         raise TypeError(f"Invalid exception type {exc}")
     elif exc_origin is Literal:
         while isinstance(exc, TypeAliasType):
@@ -61,19 +66,24 @@ def parse_exception(
         else:
             exc_local = exc
 
-        if issubclass(exc_origin, DetailBase) or issubclass(exc_local, DetailBase):
+        if lenient_issubclass(exc_origin, DetailBase) or lenient_issubclass(
+            exc_local, DetailBase
+        ):
             # if exc is a subclass of DetailBase then tha
             return cast(type["DetailBase[Any]"], exc_origin)
         raise TypeError(f"Invalid exception type {exc}")
 
 
+DExc = TypeVar("DExc", bound=DetailBase[Any] | http_status.Status)
+
+
 def __erresp_factory_registry():
     # TODO: handler can annoate return with Resp[Response, 404]
     exc_handlers: dict[type[DetailBase[Any]], ExceptionHandler[Any]] = {}
     status_handlers: dict[int, ExceptionHandler[Any]] = {}
 
-    def _extract_exception[Exc: DetailBase[Any] | http_status.Status](
-        handler: ExceptionHandler[Exc],
+    def _extract_exception(
+        handler: ExceptionHandler[DExc],
     ) -> type[DetailBase[Any]] | int | list[type[DetailBase[Any]] | int]:
         sig = signature(handler)
         _, exc = sig.parameters.values()
@@ -84,9 +94,9 @@ def __erresp_factory_registry():
 
         return parse_exception(exc_annt)
 
-    def _solver[Exc: DetailBase[Any] | http_status.Status](
-        handler: ExceptionHandler[Exc],
-    ) -> ExceptionHandler[Exc]:
+    def _solver(
+        handler: ExceptionHandler[DExc],
+    ) -> ExceptionHandler[DExc]:
         """\
         >>>
         @solver
@@ -119,7 +129,7 @@ def __erresp_factory_registry():
             return status_handlers.get(exc)
         elif isinstance(exc, TypeAliasType):
             return status_handlers.get(http_status.code(exc))
-        elif lhl_get_origin(exc) is Literal:
+        elif get_origin(exc) is Literal:
             scode: int = get_args(exc)[0]
             return status_handlers.get(scode)
 
@@ -145,13 +155,12 @@ def __erresp_factory_registry():
 
     # TODO: default ws catch
     _solver(default_error_catch)
-
     return MappingProxyType(exc_handlers), _solver, get_solver
 
 
-class HTTPException[T](Exception, DetailBase[T]):
+class HTTPException(Exception, DetailBase[T]):
     """
-    Something Wrong with the client client.
+    Something Wrong with the client request.
     """
 
     __status__: ClassVar[http_status.Status] = http_status.code(
@@ -211,7 +220,9 @@ class HTTPException[T](Exception, DetailBase[T]):
         )
 
 
-class ErrorResponse[T](Response):
+class ErrorResponse(Response, Generic[T]):
+    encoder = None
+
     def __init__(
         self,
         detail: ProblemDetail[T],
@@ -219,7 +230,9 @@ class ErrorResponse[T](Response):
         headers: Mapping[str, str] | None = None,
         media_type: str = "application/problem+json",
     ):
-        content = encode_json(detail)
+        if self.encoder is None:
+            self.encoder = encoder_factory(type(detail))
+        content = self.encoder(detail)
         super().__init__(
             content, status_code=status_code, headers=headers, media_type=media_type
         )
@@ -229,13 +242,13 @@ LIHIL_ERRESP_REGISTRY, problem_solver, get_solver = __erresp_factory_registry()
 del __erresp_factory_registry
 
 
-class CustomValidationError[T](HTTPException[T]):
+class CustomValidationError(HTTPException[T]):
     detail: str = "custom decoding errro"
 
 
 # ================== Data Validtion ================
 class ValidationProblem(Record):
-    location: ParamLocation | Literal["body"]
+    location: ParamSource
     param: str
     message: str
 
@@ -252,14 +265,16 @@ class InvalidDataType(ValidationProblem, tag=True):
     message: str = "Param is not of right type"
 
 
+class InvalidFormError(ValidationProblem, tag=True):
+    message: str = "Form is not valid"
+
+
 class CustomDecodeErrorMessage(ValidationProblem, tag=True):
     message: str
 
 
 class InvalidRequestErrors(HTTPException[list[ValidationProblem]]):
-    title: str = "Check Your Params"
-    instance: str = "URI of the entity"
-    detail: list[ValidationProblem]
+    "Check Your Params"
 
 
 # ================== Data Validtion ================
