@@ -55,9 +55,11 @@ from lihil.utils.string import (
 from lihil.utils.threading import async_wrapper
 from lihil.vendors import Request, Response
 
+DepNode = Union[IDependent[Any], tuple[IDependent[Any], INodeConfig]]
+
 
 class IEndpointProps(TypedDict, total=False):
-    errors: Sequence[type[DetailBase[Any]]] | type[DetailBase[Any]]
+    problems: Sequence[type[DetailBase[Any]]] | type[DetailBase[Any]]
     "Errors that might be raised from the current `endpoint`. These will be treated as responses and displayed in OpenAPI documentation."
     in_schema: bool
     "Whether to include this endpoint inside openapi docs"
@@ -67,33 +69,36 @@ class IEndpointProps(TypedDict, total=False):
     "Whether current endpoint should be scoped"
     auth_scheme: AuthBase | None
     "Auth Scheme for access control"
-    tags: Sequence[str] | None
+    tags: list[str] | None
     "OAS tag, endpoints with the same tag will be grouped together"
-    encoder: IEncoder
+    encoder: IEncoder | None
     "Return Encoder"
     plugins: list[IPlugin]
     "Decorators to decorate the endpoint function"
+    deps: list[DepNode] | None
+    "Dependencies that might be used in "
 
 
 class EndpointProps(Record, kw_only=True):
-    errors: tuple[type[DetailBase[Any]], ...] = field(default_factory=tuple)
+    problems: list[type[DetailBase[Any]]] = field(
+        default_factory=list[type[DetailBase[Any]]]
+    )
     to_thread: bool = True
     in_schema: bool = True
     scoped: Literal[True] | None = None
     auth_scheme: AuthBase | None = None
-    tags: Sequence[str] | None = None
+    tags: list[str] | None = None
     encoder: IEncoder | None = None
     plugins: list[IPlugin] = field(default_factory=list[IPlugin])
+    deps: list[DepNode] | None = None
 
     @classmethod
     def from_unpack(cls, **iconfig: Unpack[IEndpointProps]):
-        if raw_errors := iconfig.get("errors"):
-            if not isinstance(raw_errors, Sequence):
-                errors = (raw_errors,)
-            else:
-                errors = tuple(raw_errors)
+        if problems := iconfig.get("problems"):
+            if not isinstance(problems, Sequence):
+                problems = [problems]
 
-            iconfig["errors"] = errors
+            iconfig["problems"] = problems
         return cls(**iconfig)  # type: ignore
 
 
@@ -176,7 +181,7 @@ class Endpoint(Generic[R]):
             seen.add(decor_id)
         return func
 
-    def setup(self, sig: EndpointSignature[R], graph: Graph) -> None:
+    def _setup(self, sig: EndpointSignature[R], graph: Graph) -> None:
         if self._is_setup:
             raise Exception(f"`setup` is called more than once in {self}")
 
@@ -352,10 +357,9 @@ class Route(RouteBase):
     def __init__(
         self,
         path: str = "",
-        *,
         graph: Graph | None = None,
         middlewares: list[MiddlewareFactory[Any]] | None = None,
-        props: EndpointProps | None = None,
+        **iprops: Unpack[IEndpointProps],
     ):
         super().__init__(
             path,
@@ -364,12 +368,14 @@ class Route(RouteBase):
         )
         self._endpoints: dict[HTTP_METHODS, Endpoint[Any]] = {}
         self._call_stacks: dict[HTTP_METHODS, ASGIApp] = {}
-        if props is not None:
-            if props.tags is None:
-                props = props.replace(tags=generate_route_tag(self._path))
-            self._props = props
+
+        if iprops:
+            self._props = EndpointProps.from_unpack(**iprops)
         else:
-            self._props = EndpointProps(tags=[generate_route_tag(self._path)])
+            self._props = EndpointProps()
+
+        if self._props.deps:
+            self._graph.add_nodes(*self._props.deps)
 
         self._is_setup: bool = False
 
@@ -402,7 +408,7 @@ class Route(RouteBase):
             if ep.is_setup:
                 continue
             ep_sig = self.endpoint_parser.parse(ep.unwrapped_func)
-            ep.setup(ep_sig, self._graph)
+            ep._setup(ep_sig, self._graph)
             self._call_stacks[method] = self.chainup_middlewares(ep)
 
         self._is_setup = True
@@ -442,7 +448,7 @@ class Route(RouteBase):
                 path=merged_path,
                 graph=self._graph,
                 middlewares=sub.middle_factories,
-                props=sub._props,
+                **sub._props,
             )
             for method, ep in sub._endpoints.items():
                 new_sub.add_endpoint(method, func=ep.unwrapped_func, **ep.props)
@@ -459,10 +465,13 @@ class Route(RouteBase):
 
         if endpoint_props:
             new_props = EndpointProps.from_unpack(**endpoint_props)
-            props = self._props.merge(new_props)
+            props = self._props.merge(new_props, deduplicate=True)
         else:
             props = self._props
 
+        if not props.tags:
+            props = props.replace(tags=[generate_route_tag(self._path)])
+
         for method in methods:
             endpoint = Endpoint(
                 self, method=method, func=func, props=props, workers=self._workers
